=== modified file 'Makefile'
--- Makefile	2011-05-06 09:05:28 +0000
+++ Makefile	2011-05-08 17:58:08 +0000
@@ -431,7 +431,7 @@
 		)
 
 po/bzr.pot: $(PYFILES) $(DOCFILES)
-	$(PYTHON) tools/bzrgettext.py $(TRANSLATABLE_PYFILES) > po/bzr.pot
+	$(PYTHON) ./bzr export-pot > po/bzr.pot
 	echo $(TRANSLATABLE_PYFILES) | xargs \
 	  xgettext --package-name "bzr" \
 	  --msgid-bugs-address "<bazaar@canonical.com>" \

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-05-03 13:53:46 +0000
+++ bzrlib/builtins.py	2011-05-08 17:58:08 +0000
@@ -6154,6 +6154,16 @@
             self.outf.write('%s %s\n' % (path, location))
 
 
+class cmd_export_pot(Command):
+    __doc__ = """Export command helps and error messages in po format."""
+
+    hidden = True
+
+    def run(self):
+        from bzrlib.export_pot import export_pot
+        export_pot(self.outf)
+
+
 def _register_lazy_builtins():
     # register lazy builtins from other modules; called at startup and should
     # be only called once.

=== renamed file 'tools/bzrgettext.py' => 'bzrlib/export_pot.py'
--- tools/bzrgettext.py	2011-05-06 17:48:18 +0000
+++ bzrlib/export_pot.py	2011-05-08 17:58:08 +0000
@@ -1,9 +1,4 @@
-#!/usr/bin/env python
-#
-# bzrgettext - extract docstrings for Bazaar commands
-#
-# Copyright 2009 Matt Mackall <mpm@selenic.com> and others
-# Copyright 2011 Canonical Ltd
+# Copyright (C) 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,21 +14,26 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-# This script is copied from mercurial/i18n/hggettext and modified
-# for Bazaar.
-
 # The normalize function is taken from pygettext which is distributed
 # with Python under the Python License, which is GPL compatible.
 
-from __future__ import with_statement
-
 """Extract docstrings from Bazaar commands.
 """
 
-import os, sys, inspect
-
-
-def escape(s):
+import inspect
+import os
+import sys
+
+from bzrlib import (
+    commands as _mod_commands,
+    errors,
+    help_topics,
+    osutils,
+    plugin,
+    )
+
+
+def _escape(s):
     s = (s.replace('\\', '\\\\')
         .replace('\n', '\\n')
         .replace('\r', '\\r')
@@ -42,110 +42,101 @@
         )
     return s
 
-
-def normalize(s):
+def _normalize(s):
     # This converts the various Python string types into a format that
     # is appropriate for .po files, namely much closer to C style.
     lines = s.split('\n')
     if len(lines) == 1:
-        s = '"' + escape(s) + '"'
+        s = '"' + _escape(s) + '"'
     else:
         if not lines[-1]:
             del lines[-1]
             lines[-1] = lines[-1] + '\n'
-        lines = map(escape, lines)
+        lines = map(_escape, lines)
         lineterm = '\\n"\n"'
         s = '""\n"' + lineterm.join(lines) + '"'
     return s
 
 
-MSGID_FOUND = set()
+_FOUND_MSGID = None # set by entry function.
 
-def poentry(path, lineno, s, comment=None):
-    if s in MSGID_FOUND:
+def _poentry(outf, path, lineno, s, comment=None):
+    if s in _FOUND_MSGID:
         return
-    MSGID_FOUND.add(s)
+    _FOUND_MSGID.add(s)
     if comment is None:
         comment = ''
     else:
         comment = "# %s\n" % comment
-    print ('#: %s:%d\n' % (path, lineno) +
+    print >>outf, ('#: %s:%d\n' % (path, lineno) +
            comment+
-           'msgid %s\n' % normalize(s) +
+           'msgid %s\n' % _normalize(s) +
            'msgstr ""\n')
 
-def poentry_per_paragraph(path, lineno, msgid):
+def _poentry_per_paragraph(outf, path, lineno, msgid):
     paragraphs = msgid.split('\n\n')
     for p in paragraphs:
-        poentry(path, lineno, p)
+        _poentry(outf, path, lineno, p)
         lineno += p.count('\n') + 2
 
-def offset(src, doc, name, default):
+def _offset(src, doc, default):
     """Compute offset or issue a warning on stdout."""
     # Backslashes in doc appear doubled in src.
     end = src.find(doc.replace('\\', '\\\\'))
     if end == -1:
-        # This can happen if the docstring contains unnecessary escape
-        # sequences such as \" in a triple-quoted string. The problem
-        # is that \" is turned into " and so doc wont appear in src.
-        sys.stderr.write("warning: unknown offset in %s, assuming %d lines\n"
-                         % (name, default))
         return default
     else:
         return src.count('\n', 0, end)
 
 
-def importpath(path):
-    """Import a path like foo/bar/baz.py and return the baz module."""
-    if path.endswith('.py'):
-        path = path[:-3]
-    if path.endswith('/__init__'):
-        path = path[:-9]
-    path = path.replace('/', '.')
-    mod = __import__(path)
-    for comp in path.split('.')[1:]:
-        mod = getattr(mod, comp)
-    return mod
-
-def options(path, lineno, cmdklass):
-    cmd = cmdklass()
+def _command_options(outf, path, cmd):
     for name, opt in cmd.options().iteritems():
-        poentry(path, lineno, opt.help,
+        src, lineno = inspect.findsource(cmd.__class__)
+        lineno = _offset(''.join(src), opt.help, lineno)
+        _poentry(outf, path, lineno, opt.help,
                 "help of '%s' option of '%s' command" % (name, cmd.name()))
 
 
-def docstrings(path):
+def _write_command_help(outf, cmd_name, cmd):
+    path = inspect.getfile(cmd.__class__)
+    if path.endswith('.pyc'):
+        path = path[:-1]
+    path = os.path.relpath(path)
+    lineno = inspect.findsource(cmd.__class__)[1]
+    doc = inspect.getdoc(cmd)
+
+    _poentry_per_paragraph(outf, path, lineno, doc)
+    _command_options(outf, path, cmd)
+
+def _command_helps(outf):
     """Extract docstrings from path.
 
     This respects the Bazaar cmdtable/table convention and will
     only extract docstrings from functions mentioned in these tables.
     """
-    from bzrlib.commands import Command as cmd_klass
-    try:
-        mod = importpath(path)
-    except Exception as e:
-        # some module raises exception (ex. bzrlib.transport.ftp._gssapi
-        print >>sys.stderr, "Can't import %r: %s" % (path, e)
-        return
-    for name in dir(mod):
-        if not name.startswith('cmd_'):
-            continue
-        obj = getattr(mod, name)
-        try:
-            doc = obj.__doc__
-            if doc:
-                doc = inspect.cleandoc(doc)
-            else:
-                continue
-        except AttributeError:
-            continue
-        if (inspect.isclass(obj) and issubclass(obj, cmd_klass)
-                and not obj is cmd_klass):
-            lineno = inspect.findsource(obj)[1]
-            poentry_per_paragraph(path, lineno, doc)
-            options(path, lineno, obj)
-
-def bzrerrors():
+    import bzrlib.plugin
+    from glob import glob
+
+    # builtin commands
+    for cmd_name in _mod_commands.builtin_command_names():
+        command = _mod_commands.get_cmd_object(cmd_name, False)
+        _write_command_help(outf, cmd_name, command)
+
+    plugin_path = bzrlib.plugin.get_core_plugin_path()
+    core_plugins = glob(plugin_path + '/*/__init__.py')
+    core_plugins = [os.path.basename(os.path.dirname(p))
+                        for p in core_plugins]
+    # core plugins
+    for cmd_name in _mod_commands.plugin_command_names():
+        command = _mod_commands.get_cmd_object(cmd_name, False)
+        if command.plugin_name() not in core_plugins:
+            # skip non-core plugins
+            # TODO: Support extracting from third party plugins.
+            continue
+        _write_command_help(outf, cmd_name, command)
+
+
+def _error_messages(outf):
     """Extract fmt string from bzrlib.errors."""
     from bzrlib import errors
     base_klass = errors.BzrError
@@ -161,41 +152,26 @@
             continue
         fmt = getattr(klass, "_fmt", None)
         if fmt:
-            poentry('bzrlib/erros.py', inspect.findsource(klass)[1], fmt)
+            _poentry(outf, 'bzrlib/errors.py', inspect.findsource(klass)[1], fmt)
 
-def bzr_helptopics():
+def _help_topics(outf):
     from bzrlib.help_topics import topic_registry
     for key in topic_registry.keys():
         doc = topic_registry.get(key)
         if isinstance(doc, str):
-            poentry_per_paragraph(
+            _poentry_per_paragraph(
+                    outf,
                     'dummy/help_topics/'+key+'/detail.txt',
                     1, doc)
 
         summary = topic_registry.get_summary(key)
         if summary is not None:
-            poentry('dummy/help_topics/'+key+'/summary.txt',
-                    1, summary)
-
-
-def rawtext(path):
-    src = open(path).read()
-    poentry_per_paragraph(path, 1, src)
-
-
-if __name__ == "__main__":
-    # It is very important that we import the Bazaar modules from
-    # the source tree where bzrgettext is executed. Otherwise we might
-    # accidentally import and extract strings from a Bazaar
-    # installation mentioned in PYTHONPATH.
-    sys.path.insert(0, os.getcwd())
-    import bzrlib
-
-    with bzrlib.initialize():
-        for path in sys.argv[1:]:
-            if path.endswith('.txt'):
-                rawtext(path)
-            else:
-                docstrings(path)
-        bzrerrors()
-        bzr_helptopics()
+            _poentry(outf, 'dummy/help_topics/'+key+'/summary.txt',
+                     1, summary)
+
+def export_pot(outf):
+    global _FOUND_MSGID
+    _FOUND_MSGID = set()
+    _command_helps(outf)
+    _error_messages(outf)
+    _help_topics(outf)

